﻿// #######################################################################################################
// Program Name: CCR Client.exe
// Author: Brice Hilgemann
// Assistance: Professor Blahnik with TcpClient information, MSDN documentation on sockets
// Last Modified: 2/13/2010
// Status: In Progress
// Usage: Creates a client that connects to a Server and communicates asynchronously with this device by 
//        sending string information and also receives information from the server.
// #######################################################################################################

using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Client
{
    public partial class Form1 : Form
    {
        Socket ClientSocket;
        IAsyncResult Result;
        AsyncCallback CallBack;
        
        byte[] dataholder = new byte[512];

        public Form1()
        {
            InitializeComponent();   
        }

        private void Form1_Shown(object sender, EventArgs e)
        // ################################################################################################################
        // Default options set at startup when form is first shown
        // ################################################################################################################
        {
            button2.Enabled = false;                                  // Disable send button as no connection is made yet
            textBox3.Enabled = false;                                 // Disable send textbox as no connection is made yet

            listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Client started");

            Form1.ActiveForm.FormBorderStyle = FormBorderStyle.FixedSingle;
            Form1.ActiveForm.StartPosition = FormStartPosition.CenterScreen;    // Default form settings
            Form1.ActiveForm.MaximizeBox = false;
        }

        private void button1_Click(object sender, EventArgs e)        
        // ################################################################################################################
        // Connect/Disconnect button
        //   - Uses the entered IP and port to attempt to create a connection with the server
        // ################################################################################################################
        {
            if (button1.Text == "Connect")                 // User clicked Connect
            {
                IPEndPoint connection;                     // Create connection which is an endpoint for an IP

                try
                {
                    IPAddress ipAdd = IPAddress.Parse(textBox1.Text);     // Parse textbox1 to see if it's in the form of an IP
                    connection = new IPEndPoint(ipAdd, Convert.ToInt32(textBox2.Text));         // Convert this IP address and Port into an endpoint
                }
                catch
                {
                    listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Invalid IP Address or Port has been entered");
                    MessageBox.Show("Invalid IP Address or Port has been entered");             // Displays if invalid IP Address or Port
                    return;
                }

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Connection Attempt");   // Valid IP and Port, display that it will try to connect

                try
                {
                    ClientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);         // Use IP and Port for ClientSocket
                    ClientSocket.Connect(connection);

                    if (ClientSocket.Connected)                       // Should error out anyways and get through here no matter what if we're connected
                    {
                        listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Connected to Server");
                        button1.Text = "Disconnect";

                        textBox1.Enabled = false;
                        textBox2.Enabled = false;                     // Disable some connection things, enable sending items
                        button2.Enabled = true;
                        textBox3.Enabled = true;

                        HandleDataFromServer();
                    }
                }
                catch (SocketException ex)
                {
                    listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue " + ex);           // If something with socket fails
                }
                catch (Exception ex2)
                {
                    listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Connection issue " + ex2);      // If something with connection fails
                }		
            }
            else                                                      // User clicked Disconnect
            {
                Form1_FormClosing(sender, null);                      // Call FormClosing to handle disconnect, etc.
            }
        }

        public void HandleDataFromServer()
        // ################################################################################################################
        // Information received from Server
        //   - Bytes of data are now in the stream of which are sent from the server, callback was issued and this function
        //     is now called which handles the data
        // ################################################################################################################
        {
            try
            {
                if (CallBack == null)
                {
                    CallBack = new AsyncCallback(DataReceivedFromServer);
                }

                Result = ClientSocket.BeginReceive(dataholder, 0, dataholder.Length, SocketFlags.None, CallBack, ClientSocket);
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue2 " + ex);           // If something with socket fails
            }
            catch (Exception ex2)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Connection issue2 " + ex2);      // If something with connection fails
            }		
        }

        public void DataReceivedFromServer(IAsyncResult sync_result)
		{
			try
			{
                Socket TempSocket = (Socket)sync_result.AsyncState;

                int received = TempSocket.EndReceive(sync_result);

                string received_info = Encoding.ASCII.GetString(dataholder, 0, received);    // Convert received to string

                if (received_info == "NO_TRAINS")
                {
                    MessageBox.Show("No trains are currently available.  Try again later");
                    button1_Click(null, null);
                }

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - From Server " + received_info);    //Output string received

				HandleDataFromServer();
			}
            catch (SocketException ex)
            {
                //Server has disconnected
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Server has disconnected");
                MessageBox.Show("Server has disconnected - Program will now close");

                Form1_FormClosing(null, null);
            }
		}	

        private void button2_Click(object sender, EventArgs e)
        // ################################################################################################################
        // Send button
        //   - Takes information from textbox3 and sends it to the server through the stream
        // ################################################################################################################
        {
            try
            {
                if(textBox3.TextLength > 0)                                 // Make sure something is in the textbox
                {
                    byte[] data = Encoding.ASCII.GetBytes(textBox3.Text);   // Convert string information to bytes to send through stream

                    ClientSocket.Send(data);                                // Write data to stream to send to server

                    listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Sent " + textBox3.Text + " to Server");
                }
            }
            catch (Exception ex2)
            {
                MessageBox.Show(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Send issue " + ex2);    // If something with connection fails
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        // ################################################################################################################
        // Disconnect button
        //   - Handles closing connections before the application closes
        // ################################################################################################################
        {
            if (ClientSocket != null)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Closing Client");
                ClientSocket.Close();             // Close connection with server
                ClientSocket = null;
            }

            Application.Exit();
        }
    }
}
